# typed: true
# frozen_string_literal: true

module Krill

  module Base

    # @api private
    # This method is deprecated. Use ActiveRecord methods instead.
    def find(name, spec)

      if name == :project

        Sample.all.collect(&:project).uniq.sort

      else

        # Define available tables. Note, no queries should be made at this point
        tables = {
          item: Item.includes(sample: [:sample_type]).includes(:object_type),
          sample: Sample.includes(:sample_type),
          sample_type: SampleType,
          object_type: ObjectType,
          group: Group.all,
          upload: Upload
        }

        # Do the search
        rows = tables[name].where(pluralize_table_names(spec))

        return rows.reject(&:deleted?) if name == :item

        rows
      end

    end

    private

    def fix(val)

      if val.class == String
        "'#{val}'"
      else
        val
      end

    end

    def pluralize_table_names(spec)

      newspec = spec.clone

      reps =  { object_type: :object_types,
                sample: :samples,
                sample_type: :sample_types }

      spec.each do |k, v|
        if reps.key? k
          newspec.delete(k)
          newspec[reps[k]] = v
        end
      end

      newspec

    end

  end

end
